@@ -13,7 +13,9 @@ namespace Twig\Extra\Cache\TokenParser;
 
 use Twig\Error\SyntaxError;
 use Twig\Extra\Cache\Node\CacheNode;
+use Twig\Node\Expression\Filter\RawFilter;
 use Twig\Node\Node;
+use Twig\Node\PrintNode;
 use Twig\Token;
 use Twig\TokenParser\AbstractTokenParser;
 
@@ -35,18 +37,18 @@ class CacheTokenParser extends AbstractTokenParser
             switch ($k) {
                 case 'ttl':
                     if (1 !== \count($args)) {
-                        throw new SyntaxError(sprintf('The "ttl" modifier takes exactly one argument (%d given).', \count($args)), $stream->getCurrent()->getLine(), $stream->getSourceContext());
+                        throw new SyntaxError(\sprintf('The "ttl" modifier takes exactly one argument (%d given).', \count($args)), $stream->getCurrent()->getLine(), $stream->getSourceContext());
                     }
-                    $ttl = $args->getNode(0);
+                    $ttl = $args->getNode('0');
                     break;
                 case 'tags':
                     if (1 !== \count($args)) {
-                        throw new SyntaxError(sprintf('The "tags" modifier takes exactly one argument (%d given).', \count($args)), $stream->getCurrent()->getLine(), $stream->getSourceContext());
+                        throw new SyntaxError(\sprintf('The "tags" modifier takes exactly one argument (%d given).', \count($args)), $stream->getCurrent()->getLine(), $stream->getSourceContext());
                     }
-                    $tags = $args->getNode(0);
+                    $tags = $args->getNode('0');
                     break;
                 default:
-                    throw new SyntaxError(sprintf('Unknown "%s" configuration.', $k), $stream->getCurrent()->getLine(), $stream->getSourceContext());
+                    throw new SyntaxError(\sprintf('Unknown "%s" configuration.', $k), $stream->getCurrent()->getLine(), $stream->getSourceContext());
             }
         }
 
@@ -54,7 +56,9 @@ class CacheTokenParser extends AbstractTokenParser
         $body = $this->parser->subparse([$this, 'decideCacheEnd'], true);
         $stream->expect(Token::BLOCK_END_TYPE);
 
-        return new CacheNode($key, $ttl, $tags, $body, $token->getLine(), $this->getTag());
+        $body = new CacheNode($key, $ttl, $tags, $body, $token->getLine());
+
+        return new PrintNode(new RawFilter($body), $token->getLine());
     }
 
     public function decideCacheEnd(Token $token): bool
